@@ -1,4 +1,5 @@
 import abc
+import copy
 import hashlib
 import uuid
 from dataclasses import dataclass
@@ -10,6 +11,7 @@ from typing import (
     cast,
     Dict,
     Iterable,
+    List,
     Optional,
     Sequence,
     Set,
@@ -161,7 +163,7 @@ class CacheMissError(Exception):
         super().__init__(*args)
 
 
-KEYRING_APP_NAME = "com.sumnerevans.SublimeMusic"
+KEYRING_APP_NAME = "app.sublimemusic.SublimeMusic"
 
 
 class ConfigurationStore(dict):
@@ -173,20 +175,49 @@ class ConfigurationStore(dict):
 
     def __init__(self, **kwargs):
         super().__init__(**kwargs)
+        self._changed_secrets_store = {}
 
     def __repr__(self) -> str:
         values = ", ".join(f"{k}={v!r}" for k, v in sorted(self.items()))
         return f"ConfigurationStore({values})"
 
-    def get_secret(self, key: str) -> Any:
+    def clone(self) -> "ConfigurationStore":
+        configuration_store = ConfigurationStore(**copy.deepcopy(self))
+        configuration_store._changed_secrets_store = copy.deepcopy(
+            self._changed_secrets_store
+        )
+        return configuration_store
+
+    def persist_secrets(self):
+        if not keyring_imported or ConfigurationStore.MOCK:
+            return
+
+        for key, secret in self._changed_secrets_store.items():
+            try:
+                password_id = None
+                if password_type_and_id := self.get(key):
+                    if cast(List[str], password_type_and_id)[0] == "keyring":
+                        password_id = password_type_and_id[1]
+
+                if password_id is None:
+                    password_id = str(uuid.uuid4())
+
+                keyring.set_password(KEYRING_APP_NAME, password_id, secret)
+                self[key] = ["keyring", password_id]
+            except Exception:
+                return
+
+    def get_secret(self, key: str) -> Optional[str]:
         """
-        Get the secret value in the store with the given key. If the key doesn't exist
-        in the store, return the default. This will retrieve the secret from whatever is
-        configured as the underlying secret storage mechanism so you don't have to deal
-        with secret storage yourself.
+        Get the secret value in the store with the given key. This will retrieve the
+        secret from whatever is configured as the underlying secret storage mechanism so
+        you don't have to deal with secret storage yourself.
         """
+        if secret := self._changed_secrets_store.get(key):
+            return secret
+
         value = self.get(key)
-        if not isinstance(value, (tuple, list)) or len(value) != 2:
+        if not isinstance(value, list) or len(value) != 2:
             return None
 
         storage_type, storage_key = value
@@ -195,29 +226,14 @@ class ConfigurationStore(dict):
             "plaintext": lambda: storage_key,
         }[storage_type]()
 
-    def set_secret(self, key: str, value: Any = None) -> Any:
+    def set_secret(self, key: str, value: str = None):
         """
         Set the secret value of the given key in the store. This should be used for
-        things such as passwords or API tokens. This will store the secret in whatever
-        is configured as the underlying secret storage mechanism so you don't have to
-        deal with secret storage yourself.
+        things such as passwords or API tokens. When :class:`persist_secrets` is called,
+        the secrets will be stored in whatever is configured as the underlying secret
+        storage mechanism so you don't have to deal with secret storage yourself.
         """
-        if keyring_imported and not ConfigurationStore.MOCK:
-            try:
-                password_id = None
-                if password_type_and_id := self.get(key):
-                    if cast(Tuple[str, str], password_type_and_id[0]) == "keyring":
-                        password_id = password_type_and_id[1]
-
-                if password_id is None:
-                    password_id = str(uuid.uuid4())
-
-                keyring.set_password(KEYRING_APP_NAME, password_id, value)
-                self[key] = ["keyring", password_id]
-                return
-            except Exception:
-                pass
-
+        self._changed_secrets_store[key] = value
         self[key] = ["plaintext", value]
 
 
@@ -422,7 +438,6 @@ class Adapter(abc.ABC):
         Examples of values that could be provided include ``http``, ``https``, ``file``,
         or ``ftp``.
         """
-        # TODO (#189) actually use this
         return ()
 
     @property
@@ -430,18 +445,19 @@ class Adapter(abc.ABC):
         """
         Whether or not the adapter supports :class:`get_cover_art_uri`.
         """
+        return False
 
     @property
-    def can_stream(self) -> bool:
+    def can_get_song_file_uri(self) -> bool:
         """
-        Whether or not the adapter can provide a stream URI.
+        Whether or not the adapter supports :class:`get_song_file_uri`.
         """
         return False
 
     @property
-    def can_get_song_uri(self) -> bool:
+    def can_get_song_stream_uri(self) -> bool:
         """
-        Whether or not the adapter supports :class:`get_song_uri`.
+        Whether or not the adapter supports :class:`get_song_stream_uri`.
         """
         return False
 
@@ -468,7 +484,6 @@ class Adapter(abc.ABC):
 
         :returns: A set of :class:`AlbumSearchQuery.Type` objects.
         """
-        # TODO (#203): use this
         return set()
 
     @property
@@ -630,20 +645,26 @@ class Adapter(abc.ABC):
         """
         raise self._check_can_error("get_cover_art_uri")
 
-    def get_song_uri(self, song_id: str, scheme: str, stream: bool = False) -> str:
+    def get_song_file_uri(self, song_id: str, schemes: Iterable[str]) -> str:
         """
-        Get a URI for a given song.
+        Get a URI for a given song. This URI must give the full file.
 
         :param song_id: The ID of the song to get a URI for.
-        :param scheme: The URI scheme that should be returned. It is guaranteed that
-            ``scheme`` will be one of the schemes returned by
+        :param schemes: A set of URI schemes that can be returned. It is guaranteed that
+            all of the items in ``schemes`` will be one of the schemes returned by
             :class:`supported_schemes`.
-        :param stream: Whether or not the URI returned should be a stream URI. This will
-            only be ``True`` if :class:`supports_streaming` returns ``True``.
         :returns: The URI for the given song.
         """
-        # TODO (#189)
-        raise self._check_can_error("get_song_uri")
+        raise self._check_can_error("get_song_file_uri")
+
+    def get_song_stream_uri(self, song_id: str) -> str:
+        """
+        Get a URI for streaming the given song.
+
+        :param song_id: The ID of the song to get the stream URI for.
+        :returns: the stream URI for the given song.
+        """
+        raise self._check_can_error("get_song_stream_uri")
 
     def get_song_details(self, song_id: str) -> Song:
         """
@@ -823,6 +844,7 @@ class CachingAdapter(Adapter):
         ARTISTS = "artists"
         COVER_ART_FILE = "cover_art_file"
         DIRECTORY = "directory"
+        GENRE = "genre"
         GENRES = "genres"
         IGNORED_ARTICLES = "ignored_articles"
         PLAYLIST_DETAILS = "get_playlist_details"
