@@ -27,6 +27,8 @@ import requests
 import semver
 from gi.repository import Gtk
 
+from sublime.util import resolve_path
+
 from .api_objects import Directory, Response
 from .. import (
     Adapter,
@@ -85,7 +87,7 @@ class SubsonicAdapter(Adapter):
             name="Subsonic",
             description="Connect to a Subsonic-compatible server",
             icon_basename="subsonic",
-            icon_dir=Path(__file__).parent.joinpath("icons"),
+            icon_dir=resolve_path("adapters/subsonic/icons"),
         )
 
     @staticmethod
@@ -354,7 +356,6 @@ class SubsonicAdapter(Adapter):
         self,
         url: str,
         timeout: Union[float, Tuple[float, float], None] = None,
-        # TODO (#122): retry count
         is_exponential_backoff_ping: bool = False,
         **params,
     ) -> Any:
@@ -390,7 +391,6 @@ class SubsonicAdapter(Adapter):
                     url, params=params, verify=self.verify_cert, timeout=timeout,
                 )
 
-            # TODO (#122): make better
             if result.status_code != 200:
                 raise ServerError(
                     result.status_code, f"{url} returned status={result.status_code}."
@@ -433,7 +433,6 @@ class SubsonicAdapter(Adapter):
         )
         subsonic_response = result.json().get("subsonic-response")
 
-        # TODO (#122): make better
         if not subsonic_response:
             raise ServerError(500, f"{url} returned invalid JSON.")
 
@@ -487,7 +486,6 @@ class SubsonicAdapter(Adapter):
 
     def get_playlist_details(self, playlist_id: str) -> API.Playlist:
         result = self._get_json(self._make_url("getPlaylist"), id=playlist_id).playlist
-        # TODO (#122) better error (here and elsewhere)
         assert result, f"Error getting playlist {playlist_id}"
         return result
 
