@@ -230,6 +230,12 @@ class AlbumsPanel(Gtk.Box):
     def update(self, app_config: AppConfiguration = None, force: bool = False):
         self.updating_query = True
 
+        supported_type_strings = {
+            _to_type(t) for t in AdapterManager.get_supported_artist_query_types()
+        }
+        for i, el in enumerate(self.sort_type_combo_store):
+            self.sort_type_combo_store[i][2] = el[0] in supported_type_strings
+
         # (En|Dis)able getting genres.
         self.sort_type_combo_store[1][2] = AdapterManager.can_get_genres()
 
@@ -833,8 +839,8 @@ class AlbumsGrid(Gtk.Overlay):
             artwork.set_from_file(filename.result())
             artwork.set_loading(False)
 
-        cover_art_filename_future = AdapterManager.get_cover_art_filename(
-            item.album.cover_art
+        cover_art_filename_future = AdapterManager.get_cover_art_uri(
+            item.album.cover_art, "file"
         )
         if cover_art_filename_future.data_is_available:
             on_artwork_downloaded(cover_art_filename_future)
