@@ -56,6 +56,10 @@ class EditPlaylistDialog(Gtk.Dialog):
         self.public_switch = Gtk.Switch(active=playlist.public, halign=Gtk.Align.START)
         content_grid.attach(self.public_switch, 1, 2, 1, 1)
 
+        delete_button = Gtk.Button(label="Delete")
+        delete_button.connect("clicked", lambda *a: self.response(Gtk.ResponseType.NO))
+        content_grid.attach(delete_button, 0, 3, 1, 2)
+
         content_area.add(content_grid)
         self.show_all()
 
@@ -525,7 +529,7 @@ class PlaylistDetailPanel(Gtk.Overlay):
     @util.async_callback(
         AdapterManager.get_playlist_details,
         before_download=lambda self: self.show_loading_all(),
-        on_failure=lambda self, e: self.playlist_view_loading_box.hide(),
+        on_failure=lambda self, e: self.hide_loading_all(),
     )
     def update_playlist_view(
         self,
@@ -857,6 +861,12 @@ class PlaylistDetailPanel(Gtk.Overlay):
                 tree,
                 self.offline_mode,
                 on_download_state_change=on_download_state_change,
+                on_remove_downloads_click=(
+                    lambda: (
+                        self.offline_mode
+                        and self.playlist_songs.get_selection().unselect_all()
+                    )
+                ),
                 extra_menu_items=[
                     (
                         Gtk.ModelButton(
@@ -894,6 +904,10 @@ class PlaylistDetailPanel(Gtk.Overlay):
         self.playlist_artwork.set_loading(True)
         self.playlist_view_loading_box.show_all()
 
+    def hide_loading_all(self):
+        self.playlist_artwork.set_loading(False)
+        self.playlist_view_loading_box.hide()
+
     def make_label(self, text: str = None, name: str = None, **params,) -> Gtk.Label:
         return Gtk.Label(
             label=text,
